@@ -220,7 +220,7 @@ class numecaParser(OrderedDict):
             self.stringData[0] = "GEOMETRY-TURBO\n"
         self['ROOT'], temp = self.deepTree(self.stringData, 0)
         self['ROOT'] = self.deep_reorder_zrcurve(self['ROOT'])
-        self.merge_ZR()
+        #self.merge_ZR()
 
     def deep_reorder_zrcurve(self, group):
         new_group = iecGroup()
@@ -668,15 +668,48 @@ class numecaParser(OrderedDict):
             del self["ROOT"]["GEOMTURBO"]["CHANNEL_0"][key_to_remove]
             del self["ROOT"]["GEOMTURBO"]["CHANNEL_0"][curve[0]][curve[1]]
 
-    def exportZRNpyArrays(self):
+    def extractChannelCurves(self, channel_curve_name):
         basic_curve_dict = self.get_basic_curve_dict()
-        hub_curve_name = self["ROOT"]["GEOMTURBO"]["CHANNEL_0"]["channel_curve_hub_0"]["VERTEX"].value[0]
-        shroud_curve_name = self["ROOT"]["GEOMTURBO"]["CHANNEL_0"]["channel_curve_shroud_0"]["VERTEX"].value[0]
-        hub_curve = basic_curve_dict[hub_curve_name][1]
-        shroud_curve = basic_curve_dict[shroud_curve_name][1]
-        hub_section    = np.vstack([np.zeros(hub_curve.numberOfPoints), hub_curve.R, hub_curve.Z, np.zeros(hub_curve.numberOfPoints)]).transpose()
-        shroud_section = np.vstack([np.zeros(shroud_curve.numberOfPoints), shroud_curve.R, shroud_curve.Z, np.ones(shroud_curve.numberOfPoints)]).transpose()
-        return (np.expand_dims(hub_section,axis=0), np.expand_dims(shroud_section,axis=0))
+        # channel_curve_name possible values = channel_curve_hub_0
+        #                                      channel_curve_shroud_0
+        curve_array_list = []
+        avoid_first_occurrence = True
+        for key, item in self["ROOT"]["GEOMTURBO"]["CHANNEL_0"][channel_curve_name].items():
+            if "VERTEX" in key:
+                if not avoid_first_occurrence:
+                    curve_name = item.value[0]
+                    curve = basic_curve_dict[curve_name][1]
+                    curve_array = np.vstack([np.zeros(curve.numberOfPoints), curve.R, curve.Z, np.zeros(curve.numberOfPoints)]).transpose()
+                    curve_array = np.expand_dims(curve_array,axis=0)
+                    curve_array_list.append(curve_array)
+                else:
+                    avoid_first_occurrence = False
+        return (curve_array_list)
+
+    def exportZRNpyArraysList(self):
+        hub_array_list = self.extractChannelCurves("channel_curve_hub_0")
+        shroud_array_list = self.extractChannelCurves("channel_curve_shroud_0")
+        return (hub_array_list, shroud_array_list)
+
+    def vStackArray(self, arrayList):
+        vStackArray = arrayList[0]
+        for array in arrayList[1:]:
+            if array.shape[1] == 2:
+                array = self.fillZRArray(array, 50)
+            vStackArray = np.append(vStackArray, array[:,1:,:], axis=1)
+        return vStackArray
+
+    def fillZRArray(self, array, nPoints):
+        ref = np.array([0,1])
+        X = np.linspace(0,1,num=nPoints)
+        R = np.interp(X, ref, array[0,:,1])
+        Z = np.interp(X, ref, array[0,:,2])
+        newArray = np.vstack([np.zeros(nPoints), R, Z,np.zeros(nPoints)]).transpose()
+        return np.expand_dims(newArray,axis=0)
+
+    def exportZRNpyArrays(self):
+        hub_array_list, shroud_array_list = self.exportZRNpyArraysList()
+        return (self.vStackArray(hub_array_list), self.vStackArray(shroud_array_list))
 
     def importZRNpyArrays(self, hub_section, shroud_section):
         self.importZRNpyHubArray(hub_section)
